/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2002-2006  Charles Kerr <charles@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef PAN_TREE_STORE_H
#define PAN_TREE_STORE_H

#include <deque>
#include <vector>
#include <gtk/gtk.h>
#include <pan/data/data.h>

#define PAN_TREE_STORE_TYPE             (pan_tree_store_get_type ())
#define PAN_TREE_STORE(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), PAN_TREE_STORE_TYPE, PanTreeStore))
#define PAN_TREE_STORE_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass),  PAN_TREE_STORE_TYPE, PanTreeStoreClass))
#define IS_PAN_TREE_STORE(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PAN_TREE_STORE_TYPE))
#define IS_PAN_TREE_STORE__CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  PAN_TREE_STORE_TYPE))
#define PAN_TREE_STORE_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj),  PAN_TREE_STORE_TYPE, PanTreeStoreClass))

/**
 * Basically, a faster GtkTreeStore
 */
struct PanTreeStore
{
  public:
    GObject parent;

  public:
    struct SortInfo {
      GtkTreeIterCompareFunc sort_func;
      gpointer user_data;
      GtkDestroyNotify destroy_notify;

      SortInfo(): sort_func(0), user_data(0), destroy_notify(0) {}
      ~SortInfo() { clear(); }
      void clear () {
        if (destroy_notify) destroy_notify (user_data);
        sort_func = 0;
        user_data = 0;
        destroy_notify = 0;
      }
      void assign (GtkTreeIterCompareFunc sort_func, gpointer user_data, GtkDestroyNotify destroy_notify) {
        clear ();
        this->sort_func = sort_func;
        this->user_data = user_data;
        this->destroy_notify = destroy_notify;
      }
    };
    std::map<int,SortInfo> * sort_info; // sort info for any column
    int sort_column_id; // one of GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, or [0...n_columns)
    GtkSortType order; // the model's sort order -- ascending or descending

  public:
    struct Node {
      Node * parent; // parent Node in the tree
      int n_children; // equivalent to children.size()... hotspot
      std::vector<Node*> children; // child nodes in the tree
      int child_index; // this node is its parent's Nth child (zero-based)
      GValue * columns; // this row's data values.  This is a pointer to tree->column_types.size() GValues.
      Node(): parent(0), n_children(0), child_index(-1), columns(0) {}
    };
    int stamp; // used to verify that gtkiters came from this tree
    int n_columns; // number of columns in the tree
    Node * root; // root node
    std::deque<Node> * nodes; // acts as a mem chunk
    std::deque<Node*> * reuse_me; // nodes of deleted articles, waiting to be reused
    std::vector<GType> * column_types;
};

struct PanTreeStoreClass
{
  GObjectClass parent_class;
};


GType     pan_tree_store_get_type         (void);
PanTreeStore*  pan_tree_store_new              (int n_cols, ...);

// pan_tree_store_append() is the fastest of the various insertion methods
void      pan_tree_store_append           (PanTreeStore*, GtkTreeIter *iter, GtkTreeIter *parent);
void      pan_tree_store_insert           (PanTreeStore*, GtkTreeIter *iter, GtkTreeIter *parent, gint position);
void      pan_tree_store_insert_before    (PanTreeStore*, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling);
void      pan_tree_store_insert_after     (PanTreeStore*, GtkTreeIter *iter, GtkTreeIter *parent, GtkTreeIter *sibling);
void      pan_tree_store_prepend          (PanTreeStore*, GtkTreeIter *iter, GtkTreeIter *parent);

void      pan_tree_store_set              (PanTreeStore*, GtkTreeIter *iter, ...);
void      pan_tree_store_set_valist       (PanTreeStore*, GtkTreeIter *iter, va_list);

gboolean  pan_tree_store_remove           (PanTreeStore*, GtkTreeIter *iter);

#endif
